#!/bin/bash
#set -x

helpMenu() {
  echo "Usage:"
  echo "config_hue_iop.sh -help"
  echo "config_hue_iop.sh -ambariuser=admin -ambaripassword=admin -ambariserver=hostname.abc.com -ambariport=8080 -services=HDFS"
  echo "config_hue_iop.sh -ambariuser=admin -ambaripassword=admin -ambariserver=hostname.abc.com -ambariport=8080 -services=HDFS,YARN,HIVE"
  echo "config_hue_iop.sh -ambariuser=admin -ambaripassword=admin -ambariserver=hostname.abc.com -ambariport=8080 -services=ALL"
}

setup() {
  output=""
  hueServices=""
  currentDate=$(date +"%Y%m%d_%H%M")

  hueLog="hue_${currentDate}.log"
  echo "Logging output to $hueLog" | tee -a "$hueLog"
  hueHost=$(hostname --fqdn)
 
  #Verify user input parameters 
  curlCmd="curl -s -u $ambariuser:$ambaripassword $ambariserver:$ambariport/api/v1/clusters/"
  curlOut=$(curl -sS -u $ambariuser:$ambaripassword $ambariserver:$ambariport/api/v1/clusters)
  if [[ $curlOut == *"cluster_name"* ]]; then
    clustername=$(echo "$curlOut" |grep cluster_name | sed 's/^.*\://' |sed 's/[^"]*"\([^"]*\)".*/\1/')
    echo "Successfully connected to Ambari cluster at $ambariserver:$ambariport using credentials $ambariuser:$ambaripassword" >> "$hueLog"
    curlCmd="${curlCmd}${clustername}/"
  else
    echo "$curlOut" | tee -a "$hueLog"
    echo -e "Unable to access Ambari at $ambariserver:$ambariport using credentials $ambariuser:$ambaripassword \nplease verify Ambari username, password, server URL and port provided" >> "$hueLog"
    exit
  fi

  hueini="/usr/local/hue/desktop/conf/hue.ini"

  while true; do
    read -p "Use default hue.ini configuration file $hueini ? y/n (y) " defaultini
    case $defaultini in
      "")
        break;;
      [Yy] )
        break;;
      [Nn] )
        read -p "Enter location to hue.ini: " newlocation
        hueini=$newlocation
        break;;
      *)
        echo "Please choose [y]es or [n]o "
        ;;
    esac
  done

  echo "Creating backup of hue.ini...hue.ini_${currentDate}.bak" | tee -a "$hueLog"
  cp $hueini "hue.ini_${currentDate}.bak"

  output="${output}Configuring hue.ini at ${hueini}\n"

  #ensure port number is between 1024-65535
  validPortRegex="^(6553[0-5]|655[0-2][0-9]|65[0-4][0-9][0-9]|6[0-4][0-9]{3}|102[4-9]|10[3-9][0-9]|1[1-9][0-9][0-9]|[2-9][0-9]{3}|[2-5][0-9]{4})$"

  installedServices=$($curlCmd"services" |grep service_name | sed 's/.*: //' | sed 's/"//g')
  echo "Ambari Services Installed:" >> "$hueLog"
  echo $installedServices >> "$hueLog"
}

blacklist() {
  #Update hue.ini configuration file
  blacklistapps="impala,sqoop"
  sed -i -e "/app_blacklist/s/[^=]*$/$blacklistapps/"  -e "/app_blacklist/s/##//" ${hueini}
  output="${output}Blacklisted Applications: $blacklistapps\n"
}

genDesktopSecretKey() {
  key=$(cat $hueini |grep "secret_key=" |sed s'/^[^=]*=//')
  if [[ -z "${key// }" ]]; then
    echo "Generate random long string for Hue desktop.secret_key" | tee -a "$hueLog"
    key=$(cat /dev/urandom | tr -dc 'a-zA-Z0-9' | fold -w 32 | head -n 1)
    sed -i -e "/secret_key=/s/[^=]*$/${key}/" -e "/secret_key=/s/##//" ${hueini}
    echo "Update desktop.secret_key=$key" | tee -a "$hueLog"
  else
    echo "Desktop.secret_key already set to $key" >> "$hueLog"
  fi
}

printlog() {
  echo -e $output | tee -a "$hueLog"
}

main() {
  getJSONConfigVersions
  read -a serviceArr <<<$installedServices
  for service in "${serviceArr[@]}"
  do
    case $service in
         "HBASE")
           configHBASE
           ;;
         "HDFS")
           configHDFS
           ;;
         "HIVE")
           configHIVE
           ;;
         "KERBEROS")
           configKERBEROS
           ;;
         "MAPREDUCE2")
           configMAPREDUCE2
           ;;
         "OOZIE")
           configOOZIE
           ;;
         "PIG")
           configPIG
           ;;
         "SOLR")
           configSOLR
           ;;
         "SPARK")
           configSPARK
           ;;
         "YARN")
           configYARN
           ;;
         "ZOOKEEPER")
           configZOOKEEPER
           ;;
         *)
           echo "$service is not configured in Hue, skipping..." >> $hueLog
           ;;
    esac
  done
}

getJSONConfigVersions() {
  json=$($curlCmd"configurations" |sed '/href\|cluster_name\|stack_id/d')
}

configHBASE() {
  hbaseMasterServers=$($curlCmd"services/HBASE/components/HBASE_MASTER" |grep host_name | sed 's/^.*\://' |sed 's/[^"]*"\([^"]*\)".*/\1/')
  hbaseRestServers=$($curlCmd"services/HBASE/components/HBASE_REST_SERVER" |grep host_name | sed 's/^.*\://' |sed 's/[^"]*"\([^"]*\)".*/\1/')
  hbaseRestServers=$($curlCmd"services/HBASE/components/HBASE_REGIONSERVER" |grep host_name | sed 's/^.*\://' |sed 's/[^"]*"\([^"]*\)".*/\1/')

  hbaseServers=$(printf '%s\n' "$hbaseMasterServers" "$hbaseRestServers" "$hbaseRestServers" |sort -u)

  configHbaseThrift=false

  if [[ $hbaseServers == *"$hueHost"* ]]; then
    echo "We will start the HBASE Thrift Server on this node: ($hueHost), if you would like to start it on a different host, please select (n)o"
    while true; do
      read -p "Do you want to configure and start HBASE Thrift Server now? y/n (y) " startThrift
      case $startThrift in
        "")
          configHbaseThrift=true
          break;;
        [Yy] )
          configHbaseThrift=true
          break;;
        [Nn] )
          configHbaseThrift=false
          break;;
        *)
          echo "Please choose [y]es or [n]o "
          ;;
      esac
    done

    #Prompt user for input
    clusterName="C1"
    thriftPort="9090"

    #Start HBASE thrift server
    if [ "$configHbaseThrift" == true ]; then
      #Configure HBASE thrift server server and port
      while true; do
        read -p "Use default Hue Cluster name and port number for HBASE Thrift Server? y/n (y) " defaulthbase
        case $defaulthbase in
          "")
            break;;
          [Yy] )
            echo "Using default host: $hueHost to start HBASE Thrift Server" | tee -a "$hueLog"
            echo "Using default cluster name '$clusterName' and port '$thriftPort'" | tee -a "$hueLog"
            break;;
          [Nn] )
            while true; do
              read -p "Enter a name for HBASE cluster in Hue: ($clusterName) " name
              case $name in
                "")
                  break;;
                *)
                  if [ -n "${name##+([[:space:]])}" ]; then
                    clusterName=$name
                    break
                  else
                    echo "Please enter a valid string"
                  fi
                  ;;
              esac
            done
            while true; do
              read -p "Enter a port number to start HBASE Thrift Server: ($thriftPort) " port
              if [[ $port == "" ]]; then
                break
              elif [[ $port =~ $validPortRegex ]]; then
                thriftPort=$port
                break
              else
                echo "Please enter a valid, non-privileged port number"
              fi                  
            done
            break;;
          *)
            echo "Please choose [y]es or [n]o "
            ;;
        esac
      done

      #Update hue.ini configuration file
      sed -i -e "/hbase_clusters/s/[^=]*$/($clusterName\|$hueHost:$thriftPort)/"  -e "/hbase_clusters/s/##//" ${hueini}

      hbaseOut=$(echo "($clusterName\|$hueHost:$thriftPort)" | sed 's/\\//g')
      output="${output}Updated HBASE\n"
      output="${output}  hbase_clusters=$hbaseOut\n"

      hbaseThriftPID=$(nohup /usr/iop/current/hbase-master/bin/hbase thrift start --port $thriftPort >> "$hueLog" & echo $!)
      output="${output}Started HBASE Thrift Server, pid: ${hbaseThriftPID}\n"
      output="${output}Please check $hueLog for any port conflict errors\n"
    else
      output="${output}Did not start HBASE Thrift Server on this node, please start it manually: 'nohup /usr/iop/current/hbase-master/bin/hbase thrift start --port $thriftPort'\n"
      output="${output}Then update hue.ini [hbase] hbase_clusters=(ClusterName|hostname.abc.com:$thriftPort)\n"
    fi
  else
    output="${output}Did not configure HBASE, please configure and start the HBASE Thrift Server Manually on a node where HBase Service is installed: \n"
    output="${output}  'nohup /usr/iop/current/hbase-master/bin/hbase thrift start --port $thriftPort'\n"
    output="${output}Then update hue.ini [hbase], for example hbase_clusters=(ClusterName|hostname.abc.com:9090)\n"
  fi
}

configHDFS() {
  # Get the current configuration version in Ambari for the config file
  hdfs_site_version=$(python parseJSON.py "$json" hdfs-site)
  core_site_version=$(python parseJSON.py "$json" core-site)

  #rest call to Ambari Server to get the propertys of interest, parse out unwanted lines
  #parse out the value of the key-value pair
  hdfssite=$($curlCmd"configurations?type=hdfs-site&tag=$hdfs_site_version" |sed '/dfs.webhdfs.enabled/!d' | sed 's/^.*\://' |sed 's/[^"]*"\([^"]*\)".*/\1/')
  webhdfs_url=$($curlCmd"configurations?type=hdfs-site&tag=$hdfs_site_version" |grep "dfs.namenode.http-address.*:.*:" | sed 's/^[^:]*: //' |sed 's/[^"]*"\([^"]*\)".*/\1/')
  coresite=$($curlCmd"configurations?type=core-site&tag=$core_site_version" |sed '/fs.defaultFS.*hdfs\|hadoop.security.authentication/!d')
  fsDefaultFs=`echo "$coresite" |sed '/fs.defaultFS.*hdfs/!d' | sed 's/^[^:]*: //' |sed 's/[^"]*"\([^"]*\)".*/\1/'`
  hadoopSecurity=`echo "$coresite" |sed '/hadoop.security.authentication/!d' | sed 's/^[^:]*: //' |sed 's/[^"]*"\([^"]*\)".*/\1/'`
  kerberosEnabled=true

  #Check to see if dfs.webhdfs.enabled is enabled in Ambari
  if [ $hdfssite != "true" ]; then
    output="${output}WARNING: dfs.webhdfs.enabled is false, please enable webhdfs in Ambari HDFS\n"
  fi

  HADOOP_CONF_DIR="/etc/hadoop/conf"
  if [ -d "$HADOOP_CONF_DIR" ]; then
    #parse string for sed replacement
    hadoopConf=$(echo "$HADOOP_CONF_DIR" |sed 's/[]\/$*.^|[]/\\&/g')
    sed -i -e "/hadoop_conf_dir=/s/[^=]*$/${hadoopConf}/" -e "/hadoop_conf_dir=/s/##//" ${hueini}
  else
    echo "WARNING: HADOOP_CONF_DIR does not exist at location: $HADOOP_CONF_DIR" | tee -a "$hueLog"
    echo "Some services will not work correctly unless this is set. [/hadoop_conf_dir]" | tee -a "$hueLog"
  fi

  #parse string for sed replacement
  webhdfs_url=$(echo "$webhdfs_url" |sed 's/[]\/$*.^|[]/\\&/g')
  fsDefaultFs=$(echo "$fsDefaultFs" |sed 's/[]\/$*.^|[]/\\&/g')
  
  #Update hue.ini configuration file
  sed -i -e "/fs_defaultfs/s/[^=]*$/${fsDefaultFs}/" -e "/webhdfs_url/s/[^=]*$/http:\/\/${webhdfs_url}\/webhdfs\/v1/"  -e "/fs_defaultfs/s/##//" -e "/webhdfs_url/s/##//" ${hueini}

  #Log status to console
  fsOut=$(echo "$fsDefaultFs" | sed 's/\\//g')
  webhdfsOut=$(echo "$webhdfs_url" | sed 's/\\//g')
  output="${output}Updated HDFS\n"
  output="${output}  fs_defaultfs=$fsOut \n"
  output="${output}  fs_defaultfs=$webhdfsOut \n"
}

configHIVE() {
  hive_site_version=$(python parseJSON.py "$json" hive-site)
  hiveThriftPort=$($curlCmd"configurations?type=hive-site&tag=$hive_site_version" |sed '/hive.server2.thrift.port/!d' | sed 's/^[^:]*: //' |sed 's/[^"]*"\([^"]*\)".*/\1/')
  hiveServer=$($curlCmd"services/HIVE/components/HIVE_SERVER" |grep host_name -m 1 | sed 's/^.*\://' |sed 's/[^"]*"\([^"]*\)".*/\1/')

  #parse string for sed replacement
  hiveThriftPort=$(echo "$hiveThriftPort" |sed 's/[]\/$*.^|[]/\\&/g')
  hiveServer=$(echo "$hiveServer" |sed 's/[]\/$*.^|[]/\\&/g')

  HIVE_CONF_DIR="/etc/hive/conf"
  if [ -d "$HIVE_CONF_DIR" ]; then
    #parse string for sed replacement
    hiveConf=$(echo "$HIVE_CONF_DIR" |sed 's/[]\/$*.^|[]/\\&/g')
    sed -i -e "/hive_conf_dir=/s/[^=]*$/${hiveConf}/" -e "/hive_conf_dir=/s/##//" ${hueini}
  else
    echo "WARNING: HIVE_CONF_DIR does not exist at location: $HIVE_CONF_DIR" | tee -a "$hueLog"
    echo "Hive services may not work correctly, please set this location manually [hive_conf_dir]" | tee -a "$hueLog"
  fi


  #Update hue.ini configuration file
  sed -i -e "/hive_server_host/s/[^=]*$/$hiveServer/" -e "/hive_server_port/s/[^=]*$/$hiveThriftPort/"  -e "/hive_server_host/s/##//" -e "/hive_server_port/s/##//" ${hueini}

  hivehostOut=$(echo "$hiveServer"| sed 's/\\//g')
  hiveportOut=$(echo "$hiveThriftPort"| sed 's/\\//g')
  output="${output}Updated HIVE\n"
  output="${output}  hive_server_host=$hivehostOut\n"
  output="${output}  hive_server_port=$hiveportOut\n"
}

configKERBEROS() {
  #Check if Kerberos client is installed
  if [ $kerberosEnabled == true ] && [ `echo $hadoopSecurity | tr '[:upper:]' '[:lower:]'` == "kerberos" ]; then
    #Check if Kerberos is enabled
    echo "Kerberos is enabled in the cluster, will set security_enabled=true for all services" | tee -a "$hueLog"
    echo "If/when Kerberos is disabled, update hue.ini to set security_enabled=false; run the following command: sed -i -e \"/security_enabled/s/[^=]*$/false/" >> "${hueLog}"
    sed -i -e "/security_enabled/s/[^=]*$/true/"  -e "/security_enabled/s/##//" ${hueini}
    echo "TODO:: Update hue.ini manually to configure Kerberos keytab and principal for the hue user." | tee -a "$hueLog"
    echo "  [[kerberos]]" | tee -a "$hueLog"
    echo "    hue_keytab=/etc/security/keytabs/hue.service.keytab" | tee -a "$hueLog"
    echo "    hue_principal=hue/hostname.abc.com" | tee -a "$hueLog"
    echo "    kinit_path=/path/to/kinit" | tee -a "$hueLog"
  else
    echo "Kerberos is installed but not enabled" >> "$hueLog"
  fi
}

configMAPREDUCE2() {
  mapred_site_version=$(python parseJSON.py "$json" mapred-site)
  historyServer=$($curlCmd"configurations?type=mapred-site&tag=$mapred_site_version" |sed '/mapreduce.jobhistory.webapp.address/!d' | sed 's/^[^:]*: //' |sed 's/[^"]*"\([^"]*\)".*/\1/')
  #parse string for sed replacement
  historyServer=$(echo "$historyServer" |sed 's/[]\/$*.^|[]/\\&/g')

  #Update hue.ini configuration file
  sed -i -e "/history_server_api_url/s/[^=]*$/http:\/\/$historyServer/" -e "/history_server_api_url/s/##//" ${hueini}

  historyOut=$(echo "$historyServer"| sed 's/\\//g')
  output="${output}Updated MAPREDUCE2\n"
  output="${output}  history_server_api_url=$historyOut\n"
}

configOOZIE() {
  oozie_site_version=$(python parseJSON.py "$json" oozie-site)
  oozieServer=$($curlCmd"configurations?type=oozie-site&tag=$oozie_site_version" |sed '/oozie.base.url/!d' | sed 's/^[^:]*: //' |sed 's/[^"]*"\([^"]*\)".*/\1/')

  #parse string for sed replacement
  oozieServer=$(echo "$oozieServer" |sed 's/[]\/$*.^|[]/\\&/g')

  #Update hue.ini configuration file
  sed -i -e "/oozie_url=/s/[^=]*$/$oozieServer/" -e "/oozie_url/s/##//" ${hueini}

  oozieOut=$(echo "$oozieServer"| sed 's/\\//g')
  output="${output}Updated OOZIE\n"
  output="${output}  oozie_url=$oozieOut\n"
}

configPIG() {
  piggyBank="\/usr\/iop\/current\/pig-client\/piggybank.jar"
  pigOut=$(echo "$piggyBank"| sed 's/\\//g')

  if [ -e $pigOut ]; then
    #Update hue.ini configuration file
    sed -i -e "/local_sample_dir/s/[^=]*$/$piggyBank/" -e "/local_sample_dir/s/##/i/" ${hueini}
    output="${output}Updated PIG\n"
    output="${output}  local_sample_dir=$pigOut\n"
  else
    echo "piggybank.jar does not exist at $pigOut, update this location manually [local_sample_dir]." | tee -a "$hueLog"
  fi
}

configSOLR() {
  solrenv_version=$(python parseJSON.py "$json" solr-env)

  #rest call to Ambari Server to get the propertys of interest, parse out unwanted lines
  solrenv=$($curlCmd"configurations?type=solr-env&tag=$solrenv_version" |sed '/solr_port\"\|ZOOKEEPER_CHROOT/!d')
  #parse out the value of the key-value pair
  solrPort=`echo "$solrenv" |sed '/solr_port/!d' | sed 's/^[^:]*: //' |sed 's/[^"]*"\([^"]*\)".*/\1/'`
  solrChroot=`echo "$solrenv" |sed '/ZOOKEEPER_CHROOT/!d' | sed 's/^[^:]*: //' |sed 's/[^"]*"\([^"]*\)".*/\1/'`
  solrServer=$($curlCmd"services/SOLR/components/SOLR" |grep host_name -m 1 | sed 's/^.*://' |sed 's/[^"]*"\([^"]*\)".*/\1/')

  #parse string for sed replacement
  solrServer=$(echo "$solrServer"  |sed 's/[]\/$*.^|[]/\\&/g')
  solrChroot=$(echo "$solrChroot"  |sed 's/[]\/$*.^|[]/\\&/g')

  solrUrl="http:\/\/$solrServer:$solrPort$solrChroot\/"
  #Update hue.ini configuration file
  sed -i -e "/solr_url=/s/[^=]*$/$solrUrl/" -e "/solr_url/s/##//" ${hueini}

  solrOut=$(echo "$solrUrl" |sed 's/\\//g')
  output="${output}Updated SOLR\n"
  output="${output}  solr_url=$solrOut\n"
}

configSPARK() {
  #Spark Server Properties
  sparkHistoryServer=$($curlCmd"services/SPARK/components/SPARK_JOBHISTORYSERVER" |grep host_name | sed 's/^.*\://' |sed 's/[^"]*"\([^"]*\)".*/\1/')
  sparkHistoryServer=$(echo "$sparkHistoryServer"  |sed 's/[]\/$*.^|[]/\\&/g')

  spark_defaults_version=$(python parseJSON.py "$json" spark-defaults)
  sparkHistoryPort=$($curlCmd"configurations?type=spark-defaults&tag=$spark_defaults_version" |sed '/spark.history.ui.port/!d' | sed 's/^[^:]*: //' |sed 's/[^"]*"\([^"]*\)".*/\1/')
  #parse string for sed replacement
  sparkHistoryPort=$(echo "$sparkHistoryPort" |sed 's/[]\/$*.^|[]/\\&/g')

  configLivy=false

  #Remove Impala from the notebook snippets in Spark
  sed -i -e "/\[\[\[impala\]\]\]/ s/^/##/" -e "/\[\[\[impala\]\]\]/{n;s/^/##/;n;s/^/##/}" ${hueini}

  #Start the Spark Livy Server from Hue
  while true; do
    read -p "Do you want to start the Spark Livy Server? y/n (y)" startLivy
    case $startLivy in
      "")
        configLivy=true
        break;;
      [Yy] )
        configLivy=true
        break;;
      [Nn] )
        configLivy=false
        output="${output}Did not configure Spark, please configure and start the Spark Livy Server manually with the hue user, then update hue.ini [spark] livy_server_host with the hostname and livy_server_port to the port number Livy Server is running on\n"
        break;;
      *)
        echo "Please choose [y]es or [n]o "
        ;;
    esac
  done
 
  if [ "$configLivy" == true ]; then
    #Update hue.ini configuration filie
    sparkHistoryServerURL="http://$sparkHistoryServer:$sparkHistoryPort"
    sed -i -e "/spark_history_server_url/s/[^=]*$/http:\/\/$sparkHistoryServer:$sparkHistoryPort/" -e "/spark_history_server_url/s/##//" ${hueini}

    sparkHistoryServerURLOut=$(echo "$sparkHistoryServerURL"| sed 's/\\//g')
    output="${output}Updated SPARK\n"
    output="${output}  spark_history_server_url=$sparkHistoryServerURLOut\n"

    HADOOP_CONF_DIR="/etc/hadoop/conf"
    SPARK_HOME="/usr/iop/current/spark-client"
    if [ -d "$HADOOP_CONF_DIR" ]; then
      #parse string for sed replacement
      hadoopConf=$(echo "$HADOOP_CONF_DIR" |sed 's/[]\/$*.^|[]/\\&/g')
      sed -i -e "/hadoop_conf_dir=/s/[^=]*$/${hadoopConf}/" -e "/hadoop_conf_dir=/s/##//" ${hueini}

      if [ ! -d "$SPARK_HOME" ]; then
        echo "WARNING: SPARK_HOME does not exist at location: $SPARK_HOME" | tee -a "$hueLog"
        echo "Spark Livy will not be able to start without SPARK_HOME. To start Livy server manually, run the following as the hue user, export \$HADOOP_CONF_DIR \$SPARK_HOME; nohup /usr/local/hue/build/env/bin/hue livy_server" | tee -a "$hueLog"
      else
        livyPID=$(sudo su -c "export HADOOP_CONF_DIR=$HADOOP_CONF_DIR SPARK_HOME=$SPARK_HOME; nohup /usr/local/hue/build/env/bin/hue livy_server" - hue  >> "$hueLog" & echo $!)

        #Update hue.ini configuration file
        sed -i -e "/livy_server_host/s/[^=]*$/$hueHost/" -e "/livy_server_host/s/##//" ${hueini}

        output="${output}  livy_server_host=$hueHost\n"
        output="${output}Started Hue Spark Livy Server, pid: ${livyPID}\n"
      fi
    else
      if [ ! -d "$SPARK_HOME" ]; then
        echo "WARNING: SPARK_HOME does not exist at location: $SPARK_HOME" | tee -a "$hueLog"
        echo "Spark Livy will not be able to start without SPARK_HOME." | tee -a "$hueLog"
      fi
      echo "WARNING: HADOOP_CONF_DIR does not exist at location: $HADOOP_CONF_DIR" | tee -a "$hueLog"
      echo "Spark Livy will not be able to start without HADOOP_CONF_DIR. To start Livy server manually, run the following as the hue user, export \$HADOOP_CONF_DIR \$SPARK_HOME; nohup /usr/local/hue/build/env/bin/hue livy_server" | tee -a "$hueLog"
    fi
  fi
}

configYARN() {
  # Get the current configuration version in Ambari for the config file
  yarn_site_version=$(python parseJSON.py "$json" yarn-site)

  #rest call to Ambari Server to get the propertys of interest, parse out unwanted lines
  yarnsite=$($curlCmd"configurations?type=yarn-site&tag=$yarn_site_version" |sed '/yarn.resourcemanager.webapp.address\|yarn.resourcemanager.address/!d')
  #parse out the value of the key-value pair
  resourceManagerHostUrl=`echo "$yarnsite" |sed '/yarn.resourcemanager.address/!d' | sed 's/^[^:]*: //' |sed 's/[^"]*"\([^"]*\)".*/\1/'`
  resourceManagerHost=`echo $resourceManagerHostUrl |sed 's/:[^:]*$//'`
  resourceManagerPort=`echo $resourceManagerHostUrl |sed 's/^[^:]*://'`
  resourcemanagerAPIUrl=`echo "$yarnsite" |sed '/yarn.resourcemanager.webapp.address/!d' | sed 's/^[^:]*: //' |sed 's/[^"]*"\([^"]*\)".*/\1/'`

  #parse string for sed replacement
  resourceManagerHost=$(echo "$resourceManagerHost" |sed 's/[]\/$*.^|[]/\\&/g')
  resourceManagerPort=$(echo "$resourceManagerPort" |sed 's/[]\/$*.^|[]/\\&/g')
  resourcemanagerAPIUrl=$(echo "$resourcemanagerAPIUrl" |sed 's/[]\/$*.^|[]/\\&/g')

  #Update hue.ini configuration file
  sed -i -e "/resourcemanager_host/s/[^=]*$/$resourceManagerHost/" -e "/resourcemanager_port/s/[^=]*$/$resourceManagerPort/" -e "/resourcemanager_api_url/s/[^=]*$/http:\/\/$resourcemanagerAPIUrl/" -e "/proxy_api_url/s/[^=]*$/http:\/\/$resourcemanagerAPIUrl/" -e "/resourcemanager_host/s/##//" -e "/resourcemanager_port/s/##//" -e "0,/##.*resourcemanager_api_url/s//resourcemanager_api_url/" -e "/proxy_api_url/s/##//" ${hueini}

  #Remove all escape characters to print out nicely
  resourceManagerHostOut=$(echo "$resourceManagerHost" |sed 's/\\//g')
  resourceManagerPortOut=$(echo "$resourceManagerPort" |sed 's/\\//g')
  resourcemanagerAPIUrlOut=$(echo "$resourcemanagerAPIUrl" |sed 's/\\//g')

  output="${output}Updated YARN\n"
  output="${output}  resourcemanager_host=$resourceManagerHostOut\n"
  output="${output}  resourcemanager_port=$resourceManagerPortOut\n"
  output="${output}  resourcemanager_api_url=$resourcemanagerAPIUrlOut\n"
  output="${output}  proxy_api_url=http://$resourcemanagerAPIUrlOut\n"
}

configZOOKEEPER() {
  zoocfg_version=$(python parseJSON.py "$json" zoo.cfg)
  zooPort=$($curlCmd"configurations?type=zoo.cfg&tag=$zoocfg_version" |sed '/clientPort/!d' | sed 's/^.*\://' |sed 's/[^"]*"\([^"]*\)".*/\1/')
  zooServers=$($curlCmd"services/ZOOKEEPER/components/ZOOKEEPER_SERVER" |grep host_name | sed 's/^[^:]*: //' |sed 's/[^"]*"\([^"]*\)".*/\1/')
  zooHostPorts=`echo $zooServers |sed "s/ /:${zooPort},/g"`
  zooHostPorts="${zooHostPorts}:${zooPort}"
  zooHostPorts=$(echo "$zooHostPorts" |sed 's/[]\/$*.^|[]/\\&/g')

  #Update hue.ini configuration file
  zooHostPortsOut=$(echo "$zooHostPorts" |sed 's/\\//g')
  sed -i -e "/host_ports/s/[^=]*$/${zooHostPorts}/" -e "/host_ports/s/##//" ${hueini}
  output="${output}Updated ZOOKEEPER\n"
  output="${output}  host_ports=$zooHostPortsOut\n"

  zooRest=false

  #If Zookeeper is on the same node as Hue, prompt to start Zookeeper Rest Services
  if [[  $zooServers == *"$hueHost"* ]]; then
    while true; do
      read -p "Do you want to start Zookeeper Rest Services? y/n (y) " startZoo
      case $startZoo in
        "")
          zooRest=true
          break;;
        [Yy] )
          zooRest=true
          break;;
        [Nn] )
          zooRest=false
          output="${output}Did not configure Zookeeper Rest Services, please configure and start the Zookeeper Rest Server manually, then update hue.ini [zookeeper] host_ports=hostname:port with the hostname and port number the Zookeeper Rest services is running on\n"
          break;;
        *)
          echo "Please choose [y]es or [n]o "
          ;;
      esac
    done

    #Start Zookeeper Rest Services
    if [ "$zooRest" == true ]; then
      zooPID=$(sudo su -c "java -cp /usr/iop/current/zookeeper-server/contrib/rest/*:/usr/iop/current/zookeeper-server/contrib/rest/lib/*:/usr/iop/current/zookeeper-server/zookeeper.jar:/usr/iop/current/zookeeper-server/conf/rest org.apache.zookeeper.server.jersey.RestMain" - hue >> "$hueLog" & echo $!)
      output="${output}Started Zookeeper Rest Services, pid: ${zooPID}\n"
      #Update hue.ini configuration file for Zookeeper rest_url
      sed -i -e "/rest_url/s/[^=]*$/http:\/\/$hueHost:9998/" -e "/rest_url/s/##//" ${hueini}
      output="${output}  rest_url=http://$hueHost:9998/\n"
    fi
  else
    output="${output}Please start Zookeeper Rest Services Manually on a node with Zookeeper Service installed: 'sudo su -c \"java -cp /usr/iop/current/zookeeper-server/contrib/rest/*:/usr/iop/current/zookeeper-server/contrib/rest/lib/*:/usr/iop/current/zookeeper-server/zookeeper.jar:/usr/iop/current/zookeeper-server/conf/rest org.apache.zookeeper.server.jersey.RestMain\" - hue'\n"
    output="${output}Then manually update hue.ini [zookeeper] rest_url with the hostname and port number (9998) the Rest Services are running on\n"
  fi
}


## MAIN SCRIPT ##

if [[ $# -eq 1 ]] && [[ $1 = -help ]]; then
  helpMenu
  exit
elif [[ $# -ne 5 ]]; then
  echo "Illegal number of parameters. Run with -help to see usage."
  helpMenu
  exit
else
  for value in $*
  do
    arg=$1
    if [[ $arg = -ambariuser=* ]]; then
      ambariuser=$arg
      ambariuser=${ambariuser#*=}
    elif [[ $arg = -ambaripassword=* ]]; then
      ambaripassword=$arg
      ambaripassword=${ambaripassword#*=}
    elif [[ $arg = -ambariserver=* ]]; then
      ambariserver=$arg
      ambariserver=${ambariserver#*=}
    elif [[ $arg = -ambariport=* ]]; then
      ambariport=$arg
      ambariport=${ambariport#*=}
    elif [[ $arg = -services=* ]]; then
      services=$arg
      services=${services#*=}
    elif [[ $arg = -help ]]; then
      helpMenu
      exit
    else
      unknown=$arg
      echo "Unrecognized parameter: $unknown"
      helpMenu
      exit
    fi
    shift
  done
fi

if [ "$services" = "ALL" ]; then
  setup
  blacklist
  genDesktopSecretKey
  main
  printlog
elif [ -n "$services" ]; then
  echo "Configure Hue for Services: $services"
  setup
  blacklist
  genDesktopSecretKey
  IFS=',' read -a array <<< "$services"
  for index in "${!array[@]}"
  do
    getJSONConfigVersions

    serviceName="${array[index]}"
    case $serviceName in
      HBASE)
        if [ `echo "$installedServices" |grep "$serviceName"` ]; then
          configHBASE
        else
          output="${output}HBASE is not an installed service in Ambari\n"
        fi
        ;;
      HDFS)
        if [ `echo "$installedServices" |grep "$serviceName"` ]; then
          configHDFS
        else
          output="${output}HDFS is not an installed service in Ambari\n"
        fi
        ;;
      HIVE)
        if [ `echo "$installedServices" |grep "$serviceName"` ]; then
          configHIVE
        else
          output="${output}HIVE is not an installed service in Ambari\n"
        fi
        ;;
      KERBEROS)
        if [ `echo "$installedServices" |grep "$serviceName"` ]; then
           configKERBEROS
        else
          output="${output}KERBEROS is not an installed service in Ambari\n"
        fi
        ;;
      MAPREDUCE2)
        if [ `echo "$installedServices" |grep "$serviceName"` ]; then
          configMAPREDUCE2
        else
          output="${output}MAPREDUCE2 is not an installed service in Ambari\n"
        fi
        ;;
      OOZIE)
        if [ `echo "$installedServices" |grep "$serviceName"` ]; then
          configOOZIE
        else
          output="${output}OOZIE is not an installed service in Ambari\n"
        fi
        ;;
      PIG)
        if [ `echo "$installedServices" |grep "$serviceName"` ]; then
          configPIG
        else
          output="${output}PIG is not an installed service in Ambari\n"
        fi
        ;;
      SOLR)
        if [ `echo "$installedServices" |grep "$serviceName"` ]; then
          configSOLR
        else
          output="${output}SOLR is not an installed service in Ambari\n"
        fi
        ;;
      SPARK)
        if [ `echo "$installedServices" |grep "$serviceName"` ]; then
          configSPARK
        else
          output="${output}SPARK is not an installed service in Ambari\n"
        fi
        ;;
      YARN)
        if [ `echo "$installedServices" |grep "$serviceName"` ]; then
          configYARN
        else
          output="${output}YARN is not an installed service in Ambari\n"
        fi
        ;;
      ZOOKEEPER)
        if [ `echo "$installedServices" |grep "$serviceName"` ]; then
          configZOOKEEPER
        else
          output="${output}ZOOKEEPER is not an installed service in Ambari\n"
        fi
        ;;
      *)
        echo "unknown service ${serviceName}, not configured in Hue"
        ;;
    esac
  done
  printlog
else
  echo "Please input valid Ambari Service(s) that are supported by Hue separated by a comma, no spaces"
  echo -e "Valid Services include: HBASE,HDFS,HIVE,MAPREDUCE2,OOZIE,PIG,SOLR,SPARK,YARN,ZOOKEEPER\n"
  helpMenu
fi

